% Random Snake by Harrison Tsai
% Last modified 2013/12/04
%
% main code that runs for 500 frames

% initialize snake and food

clear
clf

global x y squares snake_size food_i food_j snake_i snake_j
x = 15;
y = 15;
frames = 200;
field = zeros(x, y);
food = rand();
snake = rand();
squares = x * y;
each_square_prob = 1/squares;
square_num = 0;
snake_size = ceil(squares * .01); % size of snake is ceil(1% of the squares)
eaten = 0;
score = 0;
orientation = 'right'; % initial orientation
count = 0;
for i = 1:x
    for j = 1:y
        square_num = square_num + 1;
        square_prob_max = each_square_prob * square_num;
        square_prob_min = square_prob_max - each_square_prob;
        if (food <= square_prob_max && food > square_prob_min)
            field(i,j) = 4; % place food
            food_i = i;
            food_j = j;
        end
        if (snake <= square_prob_max && snake > square_prob_min)
            field(i,j) = 3; % place snake head
            snake_i = i;
            snake_j = j;
        end
    end
end
if (snake_size > 1)
    body = place_body(snake_i, snake_j, orientation);
    for bi=1:2:(snake_size-1)*2
        field(body(bi), body(bi+1)) = 2; % place snake body
    end
end
imagesc(field');
set(gca,'YDir','normal');
colormap('default');
title(strcat('score: ', num2str(score)));
axis equal;
axis tight;
saveas(gcf, ['snake_',sprintf('%03d',count),'.png'])
pause(.1)

% produce frames

while (count < frames)
    while (eaten ~= 1 && count < frames)
        prob = rand();
        % use pure_probability, straight_sense, or quadrant_quest
        next = pure_probability(prob, orientation); % which direction to turn
        % get coordinates of next square
        next_square = advance(snake_i, snake_j, orientation, next);
        %pause(.1); % step delay
        field(snake_i, snake_j) = 0; % snake's old location
        if (next_square(1) == food_i && next_square(2) == food_j)
            field = zeros(x, y);
            eaten = 1;
            score = score + 1;
        else
            field(food_i, food_j) = 4;
        end
        if (snake_size > 1)
            field(body(size(body, 2)-1), body(size(body, 2))) = 0;
            body([size(body, 2)-1 size(body, 2)]) = []; % update body
            body = [snake_i snake_j body]; % update body
            for bi=1:2:(snake_size-1)*2
                field(body(bi), body(bi+1)) = 2;
            end
        end
        field(next_square(1), next_square(2)) = 3; % snake head's new location
        snake_i = next_square(1);
        snake_j = next_square(2);
        orientation = next;
        count = count + 1;
        imagesc(field');
        set(gca,'YDir','normal');
        colormap('default');
        title(strcat('score: ', num2str(score)));
        axis equal;
        axis tight;
        saveas(gcf, ['snake_',sprintf('%03d',count),'.png'])
        pause(.1)
    end
    if (count < frames)
        field = zeros(x,y);
        food = rand();
        square_num = 0;
        eaten = 0;
        for i = 1:x
            for j = 1:y
                square_num = square_num + 1;
                square_prob_max = each_square_prob * square_num;
                square_prob_min = square_prob_max - each_square_prob;
                if (food <= square_prob_max && food > square_prob_min)
                    field(i,j) = 4;
                    food_i = i;
                    food_j = j;
                end
            end
        end
        if (snake_size > 1)
            for bi=1:2:(snake_size-1)*2
                field(body(bi), body(bi+1)) = 2; % snake body
            end
        end
        field(snake_i, snake_j) = 3; % snake head location
        imagesc(field');
        set(gca,'YDir','normal');
        colormap('default');
        title(strcat('score: ', num2str(score)));
        axis equal;
        axis tight;
        saveas(gcf, ['snake_',sprintf('%03d',count),'.png'])
        pause(.1)
    end
end